///////////////////////////////////////////////////////////////////////////////
// FileContents.cpp
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#include "FileContents.h"
#include "FileDatagroupSection.h"
#include <cassert>

VFileContents::VFileContents()
{
	m_Version = 0;
	m_DynamicIdx = 0;
}

VFileContents::~VFileContents()
{
	std::vector<VFileSection *>::iterator SectionIt;

	for (SectionIt = m_Sections.begin(); SectionIt != m_Sections.end(); SectionIt++)
	{
		delete *SectionIt;
		*SectionIt = NULL;
	}

	std::vector<VFileDynamicRecord *>::iterator DynamicIt;

	for (DynamicIt = m_DynamicRecs.begin(); DynamicIt != m_DynamicRecs.end(); DynamicIt++)
	{
		delete *DynamicIt;
		*DynamicIt = NULL;
	}
}

const VFileSection * VFileContents::GetSectionByName(std::string a_strTargetName) const
{
	std::vector<VFileSection *>::const_iterator SectionIt;

	for (SectionIt = m_Sections.begin(); SectionIt != m_Sections.end(); SectionIt++)
	{
		std::string strName = (*SectionIt)->GetName();

		if (strcasecmp(strName.c_str(), a_strTargetName.c_str()) == 0)
		{
			return *SectionIt;
		}
	}

	return NULL;
}

VFileSection * VFileContents::GetSectionAt(int a_Index)
{
	assert(a_Index >= 0);

	return m_Sections[a_Index];
}

bool VFileContents::AddSection(VFileSection * a_pSection)
{
	assert(a_pSection != NULL);

	// Can't just add it.. must ensure uniqueness...

	std::vector<VFileSection *>::iterator SectionIt;

	for (SectionIt = m_Sections.begin(); SectionIt != m_Sections.end(); SectionIt++)
	{
		if (strcasecmp(a_pSection->GetName().c_str(), (*SectionIt)->GetName().c_str()) == 0)
		{
			return false;
		}
	}

	// Okay... we haven't found it..

	m_Sections.push_back(a_pSection);

	return true;
}

void VFileContents::AddDynamicRecord(VFileDynamicRecord * a_pDynamic)
{
	assert(a_pDynamic != NULL);

	m_DynamicRecs.push_back(a_pDynamic);
}

int VFileContents::GetNumDatagroups() const
{
	int NumGroups = 0;
	const VFileSection * pSection = GetSectionByName(DATAGROUP_SECTION);

	if (pSection)
	{
		const VFileDatagroupSection * pDGSection = dynamic_cast<const VFileDatagroupSection *>(pSection);

		assert(pDGSection);

		NumGroups = pDGSection->GetNumRecords();
	}

	return NumGroups;
}

bool VFileContents::GetLabelsForDatagroup(int a_GroupID, std::vector<std::string> & a_rLabels) const
{
	const VFileSection * pSection = GetSectionByName(DATAGROUP_SECTION);
	bool bRetVal = false;

	if (pSection)
	{
		const VFileDatagroupSection * pDGSection = dynamic_cast<const VFileDatagroupSection *>(pSection);

		assert(pDGSection);
	
		int NumRecs = pDGSection->GetNumRecords();
		VFileDatagroupRecord * pRecord = NULL;

		for (int i = 0; i < NumRecs; i++)
		{
			const VFileDatagroupRecord * pRecord = pDGSection->GetRecordAt(i);

			if (pRecord->GetGroupID() == a_GroupID)
			{
				a_rLabels = pRecord->GetLabels();

				bRetVal = true;
			}
		}
	}

	return bRetVal;
}

int VFileContents::GetMaxFrameNum() const
{
	// I think there's a quicker way of determining this... but for now

	std::vector<VFileDynamicRecord *>::const_iterator DynIt;
	int MaxFrame = 0, CurFrame = 0;

	for (DynIt = m_DynamicRecs.begin(); DynIt != m_DynamicRecs.end(); DynIt++)
	{
		CurFrame = (*DynIt)->GetFrameNumber();

		if (CurFrame > MaxFrame)
			MaxFrame = CurFrame;
	}

	return MaxFrame;
}

int VFileContents::GetMinFrameNum() const
{
	// I think there's a quicker way of determining this... but for now

/*	std::vector<VFileDynamicRecord *>::const_iterator DynIt;
	int MinFrame = 0, CurFrame = 0;
	
	for (DynIt = m_DynamicRecs.begin(); DynIt != m_DynamicRecs.end(); DynIt++)
	{
/*		CurFrame = (*DynIt)->GetFrameNumber();

		if (CurFrame < MinFrame)
			MinFrame = CurFrame;

		MinFrame = 
	}
*/
	int MinFrame = 0;

	if (m_DynamicRecs.size() > 0)
		MinFrame = m_DynamicRecs[0]->GetFrameNumber();

	return MinFrame;
}

VFileDynamicRecord * VFileContents::GetDynamicRecord(int a_DatagroupID, int a_Frame)
{
	// This is going to be slow also... I think a GetFirst/Next scheme 
	// would be much quicker, since the format guarantees sequentiality

	std::vector<VFileDynamicRecord *>::iterator DynIt;

	for (DynIt = m_DynamicRecs.begin(); DynIt != m_DynamicRecs.end(); DynIt++)
	{
		if ((*DynIt)->GetFrameNumber() == a_Frame &&
			(*DynIt)->GetDatagroupID() == a_DatagroupID)
			return *DynIt;
	}

	return NULL;
}

VFileDynamicRecord * VFileContents::GetFirstDynamic()
{
	m_DynamicIdx = 0;

	return *(m_DynamicRecs.begin());
}

VFileDynamicRecord * VFileContents::GetNextDynamic()
{
	if (++m_DynamicIdx < m_DynamicRecs.size())
	{
		std::vector<VFileDynamicRecord *>::iterator DynIt = m_DynamicRecs.begin();

		DynIt += m_DynamicIdx;

		return *DynIt;
	}
	else
	{
		return NULL;
	}
}

VFileDynamicRecord * VFileContents::GetFirstDynamicOfGroup(int a_DatagroupID)
{
//	m_DynamicIdx[a_DatagroupID] = 0;
	m_DynamicIdx = 0;
	std::vector<VFileDynamicRecord *>::iterator DynIt;
	VFileDynamicRecord * pRetRecord = NULL;

	for (DynIt = m_DynamicRecs.begin(); DynIt != m_DynamicRecs.end(); DynIt++)
	{
		if ((*DynIt)->GetDatagroupID() == a_DatagroupID)
		{
			pRetRecord = *DynIt;
			break;
		}
	}

	return pRetRecord;
}

VFileDynamicRecord * VFileContents::GetNextDynamicOfGroup(int a_DatagroupID)
{
	std::vector<VFileDynamicRecord *>::iterator DynIt = m_DynamicRecs.begin();
	VFileDynamicRecord * pRetRecord = NULL;

	for (DynIt += ++m_DynamicIdx; DynIt != m_DynamicRecs.end(); DynIt++, m_DynamicIdx++)
	{
		if ((*DynIt)->GetDatagroupID() == a_DatagroupID)
		{
			pRetRecord = *DynIt;
			break;
		}
	}

	return pRetRecord;
}

float VFileContents::GetFrameRate(int a_GroupID) const
{
	const VFileSection * pSection = GetSectionByName(DATAGROUP_SECTION);
	float RetVal = 0.0;

	if (pSection)
	{
		const VFileDatagroupSection * pDGSection = dynamic_cast<const VFileDatagroupSection *>(pSection);

		assert(pDGSection);

		const VFileDatagroupRecord * pRecord = pDGSection->FindDatagroupByID(a_GroupID);
	
		assert(pRecord);

		if (pRecord != NULL)
		{
			RetVal = pRecord->GetFrameRate();
		}
	}

	return RetVal;
}

